#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

require 'time'

require 'softek_utils'
require 'vm_config'
require 'sidapi'

class Sidfm < ActiveRecord::Base
  unloadable

  AVAILABLE_PASSWORD_SYMBOLS = '!#$%&()-=~^|@`[]+;*:{},.?/_'
  AVAILABLE_PASSWORD_CHAR_TYPES = {:en => '英字', :num => '数字', :sym => '記号'}

  MAX_PASSWORD_PERIOD = 365

  def self.get_value(key)
    r = where(:key => key)
    r ? (r.first ? r.first.value : nil) : nil
  end

  def self.set_value(key, value)
    ret = nil
    r = where(:key => key)
    if r and r.first then
      ret = r.first.update(:value => value)
    else
      ret = self.create(:key => key, :value => value)
    end
    ret
  end

  # FIXME: zabbix_*の整理(zabbix_senderが使える最小限に)
  # FIXME: sidfmapi_*の整理（たぶん必要なくなった)
  [
   :zabbix_user, :zabbix_password, :zabbix_server, :zabbix_url,
   :sidfmapi_user, :sidfmapi_password,
   :sidfmdlm_license_key, :sidfmdlm_license_apikey,
   :sidfmdlm_register_apikey,
  ].each do |name|
    define_singleton_method(name) { get_value(name.to_s) }
    set_name = (name.to_s + '=').to_sym
    define_singleton_method(set_name) {|value| set_value(name, value) }
  end

  def self.salt
    sidfmdlm_license_key ? "SFTK#{sidfmdlm_license_key}SFTK" : 'SFTKSFTKSFTK'
  end

  def self.sidfmdlm_register_apikey_update
    self.sidfmdlm_register_apikey = SofTekUtils.hexdigest(post: self.salt)
  end

  def self.check_sidfmdlm_register_apikey(apikey)
    self.sidfmdlm_register_apikey ? self.sidfmdlm_register_apikey == apikey : false
  end

  def self.prodver_last_update
    ret = nil
    begin
      lu = get_value(:prodver_last_update)
      ret = Time.parse(lu) if lu
    rescue Exception => e
      logger.error(e.message)
      if e.backtrace then
        e.backtrace.each do |i|
          logger.error(i)
        end
      end
    end
    ret
  end

  def self.prodver_last_update=(v)
    set_value(:prodver_last_update, v.to_s)
  end

  def self.corresp_required_sri
    ret = get_value(:corresp_required_sri)
    if ret then
      ret = ret.to_i
    else
      # High
      ret = SidfmdIssueSri::VALUE_HIGH
      set_value(:corresp_required_sri, ret.to_s)
    end
    ret
  end

  def self.corresp_required_sri=(v)
    set_value(:corresp_required_sri, v.to_s)
  end

  def self.corresp_required_cvss
    ret = get_value(:corresp_required_cvss)
    if ret then
      ret = ret.to_f
    else
      # CVSS >= 9.0
      ret = 9.0
      set_value(:corresp_required_cvss, ret.to_s)
    end
    ret
  end

  def self.corresp_required_cvss=(v)
    set_value(:corresp_required_cvss, v.to_s)
  end

  def self.corresp_required_risk
    ret = get_value(:corresp_required_risk)
    if ret then
      ret = ret.to_i
    else
      # High
      ret = 3
      set_value(:corresp_required_risk, ret.to_s)
    end
    ret
  end

  def self.corresp_required_risk=(v)
    set_value(:corresp_required_risk, v.to_s)
  end

  def self.corresp_required_area
    ret = get_value(:corresp_required_area)
    if ret then
      ret = ret.to_i
    else
      # all
      ret = 2
      set_value(:corresp_required_area, ret.to_s)
    end
    ret
  end

  def self.corresp_required_area=(v)
    set_value(:corresp_required_area, v.to_s)
  end

  def self.corresp_required_month
    ret = get_value(:corresp_required_month)
    if ret then
      ret = ret.to_i
    else
      # 1 month
      ret = 1
      set_value(:corresp_required_month, ret.to_s)
    end
    ret
  end

  def self.corresp_required_month=(v)
    set_value(:corresp_required_month, v.to_s)
  end

  def self.corresp_required
    {
      :sri => self.corresp_required_sri,
      :cvss => self.corresp_required_cvss,
      :risk => self.corresp_required_risk,
      :area => self.corresp_required_area,
      :month => self.corresp_required_month,
    }
  end

  DEFAULT_MAX_RULES = 100
  DEFAULT_MAX_RULE_LOGS = 10

  def self.max_rules
    ret = get_value(:max_rules)
    if ret then
      ret = ret.to_i
    else
      ret = DEFAULT_MAX_RULES
      set_value(:max_rules, ret.to_s)
    end
    ret
  end

  def self.max_rules=(v)
    set_value(:max_rules, v ? v.to_s : DEFAULT_MAX_RULES)
  end

  def self.max_rule_logs
    ret = get_value(:max_rule_logs)
    if ret then
      ret = ret.to_i
    else
      ret = DEFAULT_MAX_RULE_LOGS
      set_value(:max_rule_logs, ret.to_s)
    end
    ret
  end

  def self.max_rule_logs=(v)
    set_value(:max_rule_logs, v ? v.to_s : DEFAULT_MAX_RULE_LOGS)
  end

  VM_CONFIG = {}

  def self.vm_config
    if VM_CONFIG.has_key?(:instance) then
      VM_CONFIG[:instance]
    else
      VM_CONFIG[:instance] = SIDfm::VMConfig.new
      VM_CONFIG[:instance]
    end
  end

  def self.sidapi(hostname = nil, hostapikey = nil)
    api = nil
    if hostname then
      api = SIDfm::SIDAPI.new(vm_config.sidapi_url, hostname, hostapikey)
    else
      api = SIDfm::SIDAPI.new(vm_config.sidapi_url)
    end
    raise ArgumentError, 'create api failure' unless api
    if block_given? then
      begin
        yield api
      ensure
        api.close
      end
    end
    api
  end

  #パスワードポリシーの項目設定・取得
  def self.set_password_available_char_types(value)
    set_value('password_available_char_types', value)
  end

  def self.get_password_available_char_types
    v = get_value 'password_available_char_types'
    v = AVAILABLE_PASSWORD_CHAR_TYPES.stringify_keys.keys.join('/') unless v
    return v
  end

  def self.set_password_available_symbols(value)
    set_value('password_available_symbols', value)
  end

  def self.get_password_available_symbols
    v = get_value 'password_available_symbols'
    v = AVAILABLE_PASSWORD_SYMBOLS unless v
    return v
  end

  def self.set_is_password_char_type_mix_needed(value)
    set_value('is_password_char_type_mix_needed', value)
  end

  def self.get_is_password_char_type_mix_needed
    v = get_value 'is_password_char_type_mix_needed'
    v = 'false' unless v
    v = (v == 'true' ? true : false) 
    return v
  end
  
  def self.set_is_permited_password_as_same_username(value)
    set_value('is_permited_password_as_same_username', value)
  end

  def self.get_is_permited_password_as_same_username
    v = get_value 'is_permited_password_as_same_username'
    v = 'false' unless v
    v = (v == 'true' ? true : false)
    return v
  end

  def self.set_is_permited_password_includes_username(value)
    set_value('is_permited_password_includes_username', value)
  end   

  def self.get_is_permited_password_includes_username 
    v = get_value('is_permited_password_includes_username')
    v = 'false' unless v
    v = (v == 'true' ? true : false)
    return v
  end

  def self.get_password_policy_settings
    ret = []
    available_char_types = get_password_available_char_types
    all_available_char_types = AVAILABLE_PASSWORD_CHAR_TYPES.stringify_keys
    available_char_types = available_char_types.split('/')
    available_char_types_values = available_char_types.map{|c_type| all_available_char_types[c_type]}.join(', ')
    available_symbols = get_password_available_symbols
    is_chars_types_mix_needed = (get_is_password_char_type_mix_needed ? '必要' : '不要')
    is_permited_password_as_same_username = (get_is_permited_password_as_same_username ? '許可' : '不許可')
    is_permited_password_includes_username =  (get_is_permited_password_includes_username ? '許可' : '不許可')
    ret << "最小文字数: #{Setting.password_min_length}文字"
    ret << "利用可能な文字種: #{available_char_types_values}"
    ret << "利用可能な記号: #{available_symbols}" if available_char_types.include?('sym')
    ret << "文字種の混在 : #{is_chars_types_mix_needed}"
    ret << "ユーザ名と同一のパスワード: #{is_permited_password_as_same_username}"
    ret << "ユーザ名を含むパスワード: #{is_permited_password_includes_username}"
    return ret
  end

  # 10分毎に1度に処理を行うホスト数
  DEFAULT_PARALLEL_HOSTS = 20
  MAX_PARALLEL_HOSTS = 60

  def self.parallel_hosts
    ret = get_value(:parallel_hosts)
    if ret then
      ret = ret.to_i
      if ret > MAX_PARALLEL_HOSTS then
        ret = MAX_PARALLEL_HOSTS
        set_value(:parallel_hosts, ret.to_s)
      end
    else
      ret = DEFAULT_PARALLEL_HOSTS
      set_value(:parallel_hosts, ret.to_s)
    end
    ret
  end

  def self.parallel_hosts=(v)
    set_value(:parallel_hosts, v ? (v <= MAX_PARALLEL_HOSTS ? v.to_s : MAX_PARALLEL_HOSTS.to_s): DEFAULT_PARALLEL_HOSTS.to_s)
  end

  # セッションタイムアウト
  # 単位：分
  DEFAULT_SESSION_TIMEOUT = 60
  MIN_SESSION_TIMEOUT = 10

  def self.session_timeout
    ret = get_value(:session_timeout)
    if ret then
      ret = ret.to_i
      if ret < MIN_SESSION_TIMEOUT then
        ret = MIN_SESSION_TIMEOUT
        set_value(:session_timeout, ret.to_s)
      end
    else
      ret = DEFAULT_SESSION_TIMEOUT
      set_value(:session_timeout, ret.to_s)
    end
    ret
  end

  def self.session_timeout=(v)
    set_value(:session_timeout, v ? (v < MIN_SESSION_TIMEOUT ? MIN_SESSION_TIMEOUT.to_s : v.to_s) : DEFAULT_SESSION_TIMEOUT)
  end

  # メール通知設定制限
  # 不許可(true)/許可(false)
  DEFAULT_MAIL_NOTIFICATION_SETTING = false

  def self.restrict_mail_notification_setting
    ret = get_value(:restrict_mail_notification_setting)
    unless ret then
      ret = 'false'
      set_value(:restrict_mail_notification_setting, ret)
    end
    'true' == ret
  end

  def self.restrict_mail_notification_setting=(v)
    set_value(:restrict_mail_notification_setting, v ? 'true' : 'false')
  end

  # 非同期になるまでの日数
  # default(3) / 2 日以上 (同期が1日毎なので、2日未満は意味がないため）
  DEFAULT_SYNCING_CHECK_SPAN = 3

  def self.syncing_check_span
    ret = get_value(:syncing_check_span)
    if ret then
      ret = ret.to_i
      unless ret >= 2 then
        ret = 2
        set_value(:syncing_check_span, ret.to_s)
      end
    else
      ret = DEFAULT_SYNCING_CHECK_SPAN
    end
    ret
  end

  def self.syncing_check_span=(v)
    c_v = (v.to_i || DEFAULT_SYNCING_CHECK_SPAN)
    c_v = 2 unless c_v >= 2
    set_value(:syncing_check_span, c_v.to_s)
  end

  DEFAULT_MAX_DOWNLOAD_SIZE = 10000
  HISTORY_MAX_DOWNLOAD_SIZE = 10000
  HISTORY_MIN_DOWNLOAD_SIZE = 1000
  CVE_MAX_DOWNLOAD_SIZE = 10000
  CVE_MIN_DOWNLOAD_SIZE = 1000

  def self.get_history_max_download_size
    v = get_value(:get_history_max_download_size)
    unless v then
      set_value(:get_history_max_download_size, DEFAULT_MAX_DOWNLOAD_SIZE)
      v = DEFAULT_MAX_DOWNLOAD_SIZE
    end
    return v.to_i
  end
 
  def self.set_history_max_download_size=(v)
    if v < HISTORY_MIN_DOWNLOAD_SIZE  then
      v = HISTORY_MIN_DOWNLOAD_SIZE 
    elsif v > HISTORY_MAX_DOWNLOAD_SIZE then
      v = HISTORY_MAX_DOWNLOAD_SIZE
    end
    set_value(:get_history_max_download_size, v) 
  end

  def self.get_cve_max_download_size
    v = get_value(:cve_max_download_size)
    unless v then
      set_value(:cve_max_download_size, DEFAULT_MAX_DOWNLOAD_SIZE)
      v = DEFAULT_MAX_DOWNLOAD_SIZE
    end
    return v.to_i
  end

  def self.set_cve_max_download_size=(v)
    if v < CVE_MIN_DOWNLOAD_SIZE  then
      v = CVE_MIN_DOWNLOAD_SIZE
    elsif v > CVE_MAX_DOWNLOAD_SIZE then
      v = CVE_MAX_DOWNLOAD_SIZE
    end
    set_value(:cve_max_download_size, v)
  end

  def self.openid_connect_settings_user_role_id
    v = get_value(:openid_connect_settings_user_role_id)
    v = '' unless v
    v
  end

  def self.openid_connect_settings_user_role_id=(v)
    set_value(:openid_connect_settings_user_role_id, v)
  end

  def self.openid_connect_settings_user_category_ids
   v = get_value(:openid_connect_settings_user_category_ids)
   v = '' unless v
   v
  end

  def self.openid_connect_settings_user_category_ids=(v)
    set_value(:openid_connect_settings_user_category_ids, v)
  end

  DEFAULT_RULE_LOG_SIZE = 100
  def self.max_rule_log_size
    v = get_value(:max_rule_log_size)
    unless v then
      set_value(:max_rule_log_size, DEFAULT_RULE_LOG_SIZE)
      v = DEFAULT_RULE_LOG_SIZE
    end
    return v.to_i
  end

  def self.max_rule_log_size=(v)
    if v < 0 then
      v = DEFAULT_RULE_LOG_SIZE
    elsif v > SidfmdRule::LOG_MAXNUM then
      v = SidfmdRule::LOG_MAXNUM
    end
    set_value(:max_rule_log_size, v)
  end

  # 脆弱性同期：影響のあるプロダクトの更新に応じて影響無を未対応に更新
  # 脆弱性同期の際に、影響のあるプロダクトの更新に応じて影響無を
  # 未対応に更新する処理の有効/無効を設定。
  # この機能の目的は、影響のあるプロダクトが更新された場合、一度影響無し
  # と判断していても再度確認しなければならないケースが存在し、そのようなケ
  # ースに対応するため。
  # デフォルトはtrue(有効) ※ バージョン2.5.0未満のバージョンで有効だったため
  DEFAULT_VMDAEMON_AFFECT_PRODUCT = true

  def self.vmdaemon_affect_product
    ret = get_value(:vmdaemon_affect_product)
    unless ret then
      ret = DEFAULT_VMDAEMON_AFFECT_PRODUCT.to_s
      set_value(:vmdaemon_affect_product, ret)
    end
    'true' == ret
  end

  def self.vmdaemon_affect_product=(v)
    set_value(:vmdaemon_affect_product, v ? 'true' : 'false')
  end

  # 脆弱性同期の周期:各ホストが脆弱性同期される周期
  # 各ホストが、前回脆弱性同期を行ってから次の同期対象になる周期を時間で
  # 指定する。
  # デフォルトは、24(1日)。
  # なお、下限は6、上限は2400(100日)とする。
  DEFAULT_VULSYNC_INTERVAL = 24
  MIN_VULSYNC_INTERVAL = 6
  MAX_VULSYNC_INTERVAL = 2400

  def self.vulsync_interval
    ret = get_value(:vulsync_interval)
    unless ret then
      ret = DEFAULT_VULSYNC_INTERVAL
      set_value(:vulsync_interval, ret.to_s)
    end
    ret.to_i
  end

  def self.vulsync_interval=(v)
    v = DEFAULT_VULSYNC_INTERVAL if (v.nil? or !v.respond_to?(:to_i))
    v = v.to_i
    v = MIN_VULSYNC_INTERVAL if v < MIN_VULSYNC_INTERVAL
    v = MAX_VULSYNC_INTERVAL if v > MAX_VULSYNC_INTERVAL
    set_value(:vulsync_interval, v.to_s)
  end

end
