#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

require File.expand_path('./base.rb', File.dirname(__FILE__))

require 'uri'
require 'json'
require 'openssl'
require 'net/http'

require 'vm_config'

module SIDfm

  class APIBase

    # 指定された時間（秒）アクセスを行わなかった場合は、次のアクセス時に再接続を行う。
    KEEPALIVE_TIMEOUT = 60

    DEBUG_LOG = '/var/lib/redmine/log/apibase-debug.log'.freeze

    # デバッグモードの有効無効切り替え
    # overwriteしてtrueを返せば有効になる。
    def self.is_debug?
      FileTest.exist?('/flag.debug')
    end

    # SSL cetificate errorを無視する設定
    def self.is_nosslverify?
      FileTest.exist?('/flag.no_sslverify')
    end

    # エラー出力
    def self.errout(label, *o)
      if o then #1
        File.open(DEBUG_LOG, 'a') do |f|
          o.each do |i| #2
            f.puts "#{label}: ", i.inspect
          end #2
        end
      end #1
    end

    # デバッグ出力
    def self.debugout(label, *o)
      errout(label, *o) if is_debug?
    end

    # デストラクタ
    def self.destructor(http: nil, status: nil)
      proc {
        if http and http.started? then #1
          begin #2
            http.finish
          rescue IOError => e #2
            errout(:destructor, e) if is_debug?
          end #2
        end #1
      }
    end

    private

    def initialize(base_url = nil)
      raise ArgumentError unless base_url
      @debug_out = File.open('/var/lib/redmine/log/http-debug.log', 'a') if self.class.is_debug?
      @vm_config = SIDfm::VMConfig.new
      @base_url = base_url
      @http = nil
      @http_status = { :host => nil, :port => nil, :time => Time.new(0) }
      ObjectSpace.define_finalizer(self, self.class.destructor(http: @http, status: @http_status))
      if block_given? then #1
        begin #2
          yield self
        ensure #2
          close
        end #2
      end #1
    end

    # 対象Webサーバへの接続
    def connect(url, force: false)
      # hostとportが一致し、KEEPALIVE_TIMEOUT秒以内に既に接続されているなら、
      # 接続を行わず終了（既に接続されているコネクションをそのまま利用）。
      # 引数forceがtrueの場合、強制的に接続し直す。
      return if
        force or
        !@http.nil? and (Time.now - @http_status[:time] <= KEEPALIVE_TIMEOUT.to_f and
                         (@http_status[:host] == url.host and
                          @http_status[:port] == url.port))
      # cert設定
      #cert_file = '/var/lib/redmine/plugins/sidfm-vm-server/ROR/config/cert.pem'
      @proxy = @vm_config.get_proxy(url.scheme == 'https')
      @http = @proxy.new(url.host, url.port)
      # http/https以外は排除
      raise ArgumentError if url.scheme !~ /^https?$/
      @http.use_ssl = (url.scheme == 'https')
      @http.set_debug_output(@debug_out) if self.class.is_debug?
      @http.verify_mode = OpenSSL::SSL::VERIFY_NONE if self.class.is_nosslverify?
      @http.open_timeout = 60
      @http.read_timeout = 60
      @http.keep_alive_timeout = KEEPALIVE_TIMEOUT
    end

    # 対象Webサーバからの切断
    def disconnect
      self.class.destructor(http: @http, status: @http_status).call
      @http = nil
    end

    # アクセス時に自動的に追加するヘッダをHashで返す。
    # overrideの際には親メソッドの値に追加／削除を行う形で実装すること。
    def create_request_headers
      { 'Content-Type' => 'application/json; charset=UTF-8' }
    end

    # URLのpathとqueryだけを抽出した文字列を返す。
    def url_path_query(url)
      url.select(:path, :query).join('?')
    end

    # params(Hash)をパラメータと見なして文字列に変換し、pathと結合する
    def path_params(path, params)
      ret = ''
      ret << path
      ret << '?'
      ret << params.collect {|k, v|
        v = '' unless v
        k ? "#{URI.encode(k.to_s)}=#{URI.encode(v.to_s)}" : "=#{URL.encode(v.to_s)}"
      }.join('&') if params.size > 0
      self.class.debugout(:path_params, ret) if self.class.is_debug?
      ret
    end

    # リクエストを送信する
    def send_request(method, path, headers: nil, body: nil)
      if body then #1
        @http.send(method, path, body, headers)
      else #1
        @http.send(method, path, headers)
      end #1
    end

    # apiアクセス
    def api_request(path, method: :get, body: nil, **params)
      url = URI.parse(@base_url + path_params(path, params))
      connect(url)
      response = send_request(method, url_path_query(url), headers: create_request_headers, body: body)
      ret = { 'success' => false }
      begin
        ret = JSON.parse(response.body, :max_nesting => false, :allow_nan => true)
      rescue JSON::ParserError => e
        ret['error'] = e.message
        ret['backtrace'] = e.backtrace
      end
      ret
    end

    public

    # 終了処理
    def close
      disconnect
      ObjectSpace.undefine_finalizer(self)
    end

  end

end
